#!/usr/bin/python
#
#  svn_adapter.py
#  Changes
#
#  Created by Ian Baird on 3/22/09.
#  Copyright (c) 2009 Skorpiostech, Inc. All rights reserved.
#

import getopt, sys, objc, xml.dom.minidom, datetime, calendar, os
from subprocess import PIPE, Popen, call
from Foundation import NSPropertyListSerialization, NSPropertyListXMLFormat_v1_0, NSDate

###############################################################################

help = """
--command=list_revs <file path>
    list revisions for a file at <file path>
    
--command=test_managed <file path>
    test a file at <file path> for management under this scm flavor
    
--command=export_rev --rev=<revision> --exporturl=exporturl <file path> 
    exports a file at <exporturl> (<file path> is not needed)

--flavor
    returns the scm flavor

--help
    display this help message

"""

def svnPath():
    if os.environ.has_key('CHNG_SVN'):
        return os.environ['CHNG_SVN']
    
    # search the usual spots
    hits = [path for path in [ '/opt/subversion/bin/svn', '/opt/local/bin/svn', '/usr/local/bin/svn', '/usr/bin/svn' ] if os.path.exists(path)]
    
    if len(hits) > 0:
        return hits[0]
    
    return "svn"


def getText(nodelist):
    """ utility method for the dom stuff
    """
    rc = ""
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data
    return rc

def list_revs(arg0):
    """ prints out a plist with the revisions for arg0
    """
    p = Popen([svnPath(), "log", "--non-interactive", "--xml", arg0], 
        bufsize=1024, stdout=PIPE, close_fds=True, stderr=PIPE)
    child_stdout = p.stdout
    child_stderr = p.stderr
    
    dom = xml.dom.minidom.parseString(child_stdout.read())
    
    returnCode = p.wait()
    
    logEntries = \
        [ 
            dict(rev=int(elem.getAttribute("revision")), 
                 author=getText(elem.getElementsByTagName("author")[0].childNodes), 
                 date=NSDate.dateWithTimeIntervalSince1970_(
                     calendar.timegm(
                         datetime.datetime.strptime(
                             getText(elem.getElementsByTagName("date")[0].childNodes).split('.')[0], 
                             "%Y-%m-%dT%H:%M:%S"
                         ).timetuple()
                     )
                 ), 
                 msg=getText(elem.getElementsByTagName("msg")[0].childNodes)) 
            for elem in dom.getElementsByTagName("logentry") 
        ]
    
    returnDict = dict(revList=logEntries, returnCode=returnCode, stdErr=child_stderr.read())
    
    child_stdout.close()
    child_stderr.close()
    
    plistData, errorString = \
        NSPropertyListSerialization.dataFromPropertyList_format_errorDescription_(returnDict, 
            NSPropertyListXMLFormat_v1_0, objc.NULL)
    sys.stdout.write(str(plistData))
    
def test_managed(arg0):
    """ tests to see if arg0 is under SCM, prints a plist containing TRUE if it does, FALSE 
        if it does not
    """
    
    try:
        p = Popen([svnPath(), "info", "--non-interactive", arg0], bufsize=1024, stdout=PIPE, stderr=PIPE,
            close_fds=True)
        child_stdout = p.stdout
        child_stderr = p.stderr
        
        returnCode = p.wait()
        
        isManaged = child_stdout.read().find("Revision:") != -1
        
        stdErr=child_stderr.read()
        
        child_stdout.close()
        child_stderr.close()
    except OSError, e:
        isManaged = False
        returnCode = e.errno
        stdErr = e.strerror
    
    returnDict = dict(isManaged=isManaged, returnCode=returnCode, stdErr=stdErr)
    
    plistData, errorString = \
        NSPropertyListSerialization.dataFromPropertyList_format_errorDescription_(returnDict, 
            NSPropertyListXMLFormat_v1_0, objc.NULL)
    sys.stdout.write(str(plistData))

def export_rev(arg0, exportURL, rev):
    """ exports a revision to a temp file, then prints the a plist containing the temp file path
    """
    import warnings
    
    # Don't need the tmpnam warning here, we're stuck with this :-(
    warnings.simplefilter("ignore", RuntimeWarning)
    svnTmpPath = os.tmpnam()
    warnings.resetwarnings()
    
    p = Popen([svnPath(), "export", "-q", "--non-interactive", "-r", rev, arg0, svnTmpPath], bufsize=1024, stdout=PIPE, 
        stderr=PIPE, close_fds=True)
    child_stdout = p.stdout
    child_stderr = p.stderr
    
    returnCode = p.wait()
    
    returnDict = dict(tmpPath=svnTmpPath, returnCode=returnCode, stdErr=child_stderr.read()) 
    
    child_stdout.close()
    child_stderr.close()
    
    plistData, errorString = \
            NSPropertyListSerialization.dataFromPropertyList_format_errorDescription_(returnDict, 
                NSPropertyListXMLFormat_v1_0, objc.NULL)
    sys.stdout.write(str(plistData))
        
def url_for_export(arg0):
    """ tests to see if arg0 is under SCM, prints a plist containing TRUE if it does, FALSE 
        if it does not
    """
    p = Popen([svnPath(), "info", "--non-interactive", arg0], bufsize=1024, stdout=PIPE, stderr=PIPE,
            close_fds=True)
    child_stdout = p.stdout
    child_stderr = p.stderr
    
    urlLine = False
    
    for line in child_stdout:
        if line.startswith('URL: '):
            urlLine = line[5:].strip()
            break
            
    returnCode = p.wait()
    
    returnDict = dict(exportURL=urlLine, returnCode=returnCode, stdErr=child_stderr.read())
    
    child_stdout.close()
    child_stderr.close()
    
    plistData, errorString = \
        NSPropertyListSerialization.dataFromPropertyList_format_errorDescription_(returnDict, 
            NSPropertyListXMLFormat_v1_0, objc.NULL)
    sys.stdout.write(str(plistData))

def flavor():
    plistData, errorString = \
        NSPropertyListSerialization.dataFromPropertyList_format_errorDescription_("Subversion", 
        NSPropertyListXMLFormat_v1_0, objc.NULL)
    sys.stdout.write(str(plistData))
    
def main():
    """ Parse and act on the options that are passed in 
    """
    global help
    optlist, args = getopt.getopt(sys.argv[1:], [], ["command=", "rev=","help","flavor","exporturl="])
    
    dispatch = dict(list_revs=list_revs, test_managed=test_managed, export_rev=export_rev, 
        url_for_export=url_for_export)
    
    command = None
    rev = None
    exportURL = None
    
    for o, a in optlist:
        if o == "--command" and dispatch.has_key(a):
            command = dispatch[a]
        elif o == "--rev":
            rev = a
        elif o == "--exporturl":
            exportURL = a
        elif o == "--flavor":
            flavor()
        elif o == "--help":
            sys.stderr.write(help)
        else:
            sys.stderr.write("Unrecognized switch: %s" % o)
    
    if command and rev and exportURL:
        command(args[0], exportURL, rev)
    elif command:
        command(args[0])
    

if __name__ == '__main__':
    main()
    